/*
 * Name:    mt_amplifier.c
 *
 * Purpose: Tcl interface to generic MX amplifier functions.
 *
 * Author:  William Lavender
 *
 *---------------------------------------------------------------------------
 *
 * Copyright 2004 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>

#include <tcl.h>

#include "mx_util.h"
#include "mx_record.h"
#include "mx_amplifier.h"
#include "mx_handle.h"
#include "mx_tcl.h"
#include "mx_tcl_private.h"

MX_EXPORT int
Mx_amplifier_get_gain_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double gain;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: amplifier_get_gain record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_amplifier_get_gain( record, &gain );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, gain );

	return TCL_OK;
}

MX_EXPORT int
Mx_amplifier_set_gain_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_amplifier_set_gain_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double gain;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: amplifier_set_gain record_handle value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &gain );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[2], "double", fname );

	mx_status = mx_amplifier_set_gain( record, gain );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_amplifier_get_offset_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double offset;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: amplifier_get_offset record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_amplifier_get_offset( record, &offset );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, offset );

	return TCL_OK;
}

MX_EXPORT int
Mx_amplifier_set_offset_cmd( ClientData clientData, Tcl_Interp *interp,
			int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_amplifier_set_offset_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double offset;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: amplifier_set_offset record_handle value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &offset );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[2], "double", fname );

	mx_status = mx_amplifier_set_offset( record, offset );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

MX_EXPORT int
Mx_amplifier_get_time_constant_cmd( ClientData clientData, Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[] )
{
	Tcl_Obj *resultPtr;
	MX_RECORD *record;
	double time_constant;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: amplifier_get_time_constant record_handle" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	mx_status = mx_amplifier_get_time_constant( record, &time_constant );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	resultPtr = Tcl_GetObjResult( interp );
	Tcl_SetDoubleObj( resultPtr, time_constant );

	return TCL_OK;
}

MX_EXPORT int
Mx_amplifier_set_time_constant_cmd( ClientData clientData, Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[] )
{
	static const char fname[] = "Mx_amplifier_set_time_constant_cmd()";

	MX_RECORD *record;
	int tcl_status;
	double time_constant;
	mx_status_type mx_status;

	if ( objc != 3 ) {
		Tcl_WrongNumArgs( interp, 2, objv,
		"Usage: amplifier_set_time_constant record_handle value" );
		return TCL_ERROR;
	}

	mx_status = mxtcl_get_record_from_object( &record, interp, objv[1] );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	tcl_status = Tcl_GetDoubleFromObj( interp, objv[2], &time_constant );

	if ( tcl_status != TCL_OK )
		return mxtcl_object_parse_error(
				interp, objv[2], "double", fname );

	mx_status = mx_amplifier_set_time_constant( record, time_constant );

	if ( mx_status.code != MXE_SUCCESS )
		return mxtcl_error( interp, mx_status );

	return TCL_OK;
}

